#include<avr/io.h>
#include"MMC_SD/MMC_SD.h"
#include"FAT/FAT.h"
#include <avr/interrupt.h>   
#include <util/delay.h>

// control line pins
#define	LCD_CD	PC7   
#define	LCD_WR  PC6
#define	LCD_RD  PC5
#define	LCD_CS	PC4
#define	LCD_RES	PC2

// macros for control lines
#define CLR_CD PORTC &= ~(1<<LCD_CD);
#define SET_CD PORTC |=  (1<<LCD_CD);

#define CLR_CS PORTC &= ~(1<<LCD_CS);
#define SET_CS PORTC |=  (1<<LCD_CS);

#define CLR_RESET PORTC &= ~(1<<LCD_RES);
#define SET_RESET PORTC |=  (1<<LCD_RES);

#define CLR_WR PORTC &= ~(1 << LCD_WR);
#define SET_WR PORTC |=  (1 << LCD_WR);
#define CLR_RD PORTC &= ~(1 << LCD_RD);
#define SET_RD PORTC |=  (1 << LCD_RD);

// color definitions
#define	BLACK	  0x0000
#define	BLUE	  0x001F
#define	RED 	  0xF800
#define	GREEN   0x07E0
#define CYAN	  0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE	  0xFFFF

#define uint8 unsigned char
#define uint16 unsigned int
#define uint32 unsigned long

unsigned char * PATH = "\\batch";

extern uint16 SectorsPerClust;
extern uint16 FirstDataSector; 
extern uint8 FAT32_Enable;

struct FileInfoStruct FileInfo;

struct direntry PictureInfo;
uint16 totalfiles = 0;

// ************************************************* //
void delay(unsigned int t)
{
	unsigned char t1;
	while(t--)
	for(t1=11;t1>0;t1--)
	{asm("nop");}
}
// ************************************************* //
void read_status()
{
	DDRE=0x00;

	CLR_CS;
	CLR_CD;
	SET_RD;
	SET_WR;

	CLR_RD;
	SET_RD;

	delay(10000);

	CLR_RD;
	SET_RD;

	SET_CS;
	SET_CD;
	DDRE=0xFF;
}
// ************************************************* //
void write_command(unsigned int command)
{
	CLR_CS;
	CLR_CD;
	SET_RD;
	SET_WR;

	PORTE=command>>8;
	CLR_WR;
	SET_WR;
	PORTE=command;

	CLR_WR;
	SET_WR;
	SET_CS;
}

// ************************************************* //
void write_data(unsigned int data)
{
	CLR_CS;
	SET_CD;
	SET_RD;
	SET_WR;

	PORTE=data>>8;
	CLR_WR;
	SET_WR;
	PORTE=data;

	CLR_WR;
	SET_WR;
	SET_CS;
}
// ************************************************* //
void initialization()
{
	SET_RD;
	SET_WR;
	SET_CS;
	SET_CD;
	PORTE=0x00;

	CLR_RESET;
	delay(10);
	delay(10);
	SET_RESET;
	delay(50);

  // ************* Start Initial Sequence ********** // 
  write_command(0x0001);
	write_data(0x0100); // set SS and SM bit
  write_command(0x0002);
	write_data(0x0200); // set 1 line inversion
  write_command(0x0003);
	write_data(0x1030); // set GRAM write direction and BGR=1.
  write_command(0x0004);
	write_data(0x0000); // Resize register
  write_command(0x0008);
	write_data(0x0207); // set the back porch and front porch
  write_command(0x0009);
	write_data(0x0000); // set non-display area refresh cycle ISC[3:0]
  write_command(0x000A);
	write_data(0x0000); // FMARK function
  write_command(0x000C);
	write_data(0x0000); // RGB interface setting
  write_command(0x000D);
	write_data(0x0000); // Frame marker Position
  write_command(0x000F);
	write_data(0x0000); // RGB interface polarity
  // *************Power On sequence **************** // 
  write_command(0x0010);
	write_data(0x0000); // SAP, BT[3:0], AP, DSTB, SLP, STB
  write_command(0x0011);
	write_data(0x0007); // DC1[2:0], DC0[2:0], VC[2:0]
	write_command(0x0012);
	write_data(0x0000); // VREG1OUT voltage
  write_command(0x0013);
	write_data(0x0000); // VDV[4:0] for VCOM amplitude
  _delay_ms(200);                        // Dis-charge capacitor power voltage
  write_command(0x0010);
	write_data(0x1690); // SAP, BT[3:0], AP, DSTB, SLP, STB
  write_command(0x0011);
	write_data(0x0227); // R11h=0x0221 at VCI=3.3V, DC1[2:0], DC0[2:0], VC[2:0]
  _delay_ms(50);                         // Delay 50ms
  write_command(0x0012);
	write_data(0x008C); // External reference voltage= Vci;
  _delay_ms(50);                         // Delay 50ms
  write_command(0x0013);
	write_data(0x1700); // VDV[4:0] for VCOM amplitude  // 0x1D00
  write_command(0x0029);
	write_data(0x002C); // VCM[5:0] for VCOMH           // 0x0035
  write_command(0x002B);
	write_data(0x000D); // Frame Rate = 93Hz
  _delay_ms(50);                         // Delay 50ms
  write_command(0x0020);
	write_data(0x0000); // GRAM horizontal Address
  write_command(0x0021);
	write_data(0x0000); // GRAM Vertical Address
  // ----------- Adjust the Gamma Curve ----------//
  write_command(0x0030);
	write_data(0x0000);
  write_command(0x0031);
	write_data(0x0507);
  write_command(0x0032);
	write_data(0x0004);
  write_command(0x0035);
	write_data(0x0205);
  write_command(0x0036);
	write_data(0x0004);
  write_command(0x0037);
	write_data(0x0307);
  write_command(0x0038);
	write_data(0x0002);
  write_command(0x0039);
	write_data(0x0707);
  write_command(0x003C);
	write_data(0x0502);
  write_command(0x003D);
	write_data(0x0005);
  //------------------ Set GRAM area ---------------//
  write_command(0x0050);
	write_data(0x0000); // Horizontal GRAM Start Address
  write_command(0x0051);
	write_data(0x00EF); // Horizontal GRAM End Address
  write_command(0x0052);
	write_data(0x0000); // Vertical GRAM Start Address
  write_command(0x0053);
	write_data(0x013F); // Vertical GRAM Start Address
  write_command(0x0060);
	write_data(0xA700); // Gate Scan Line
  write_command(0x0061);
	write_data(0x0001); // NDL,VLE, REV
  write_command(0x006A);
	write_data(0x0000); // set scrolling line
  //-------------- Partial Display Control ---------//
  write_command(0x0080);
	write_data(0x0000);
  write_command(0x0081);
	write_data(0x0000);
  write_command(0x0082);
	write_data(0x0000);
  write_command(0x0083);
	write_data(0x0000);
  write_command(0x0084);
	write_data(0x0000);
  write_command(0x0085);
	write_data(0x0000);
  //-------------- Panel Control -------------------//
  write_command(0x0090);
	write_data(0x0010);
  write_command(0x0092);
	write_data(0x0000);
  write_command(0x0093);
	write_data(0x0003);
  write_command(0x0095);
	write_data(0x0100);
  write_command(0x0097);
	write_data(0x0000);
  write_command(0x0098);
	write_data(0x0000);
  write_command(0x0007);
	write_data(0x0133); // 262K color and display ON
}


// ************************************************* //
void Display_Home()
{
	write_command( 0x0020); // GRAM Address Set (Horizontal Address) (R20h)
	write_data( 0x0000);
	write_command( 0x0021); // GRAM Address Set (Vertical Address) (R21h)
	write_data( 0x0000);
	write_command(0x0022);  // Write Data to GRAM (R22h)
}
// ************************************************* //
void display_rgb(unsigned int data)
{
	unsigned int i,j;
	Display_Home();

	for(i=0;i<320;i++)
	{
		for(j=0;j<240;j++)
		{
			write_data(data);
		}
	}
}
// ************************************************* //
void Picture_Slide_Show()
{
	uint8 i;              //loop variable
	uint16 j;             //loop variable
	uint32 p;             //cluster
	uint16 *buffer;				//buffer
	uint16 pics = 1;
	uint16 slide_show_flag = 1;
	uint8 sector;
	uint32 pixels;

	if (totalfiles == 0)
	  return;

	buffer = malloc(512);
	if (buffer == 0)
	  return;

next: 	//label for "goto"
	display_rgb(BLACK);

	Search(PATH, &PictureInfo, &pics);                                         //find the file
	p = PictureInfo.deStartCluster + (((uint32)PictureInfo.deHighClust) << 16);//the first cluster of the file

	sector = 0;

	FAT_LoadPartCluster(p, sector, buffer);	//read a sector


	pixels = (uint32) 320 * 240; // total # of pixels to write

	Display_Home();
	
	j = 0; // byte count
	
	while (1)
	{
		write_data(buffer[j]);  // write 16 bits
		pixels--;               // which is one pixel
		j++;                    // word count

		if (pixels == 0)        
		break;

		if (j == 256)           // time for a new sector     
		{
			sector++;
			if (sector == SectorsPerClust)
			{
				p = FAT_NextCluster(p);				// read next cluster
				sector = 0;
			}

			FAT_LoadPartCluster(p, sector, buffer);	// read a sector
			j = 0;
		}
	}

	if (slide_show_flag)
	{
		for (i=0;i<30;i++)	// delay for a while
		delay(0xffff);

		pics++;					// increment picture number
		if (pics > totalfiles)	// if last
		pics = 1;			// wrap around

		goto next;				// go show next pic
	}
	free(buffer);
}
// ************************************************* //
void LCD_test()
{
	unsigned int i,j;
	Display_Home();

	for(i=0;i<320;i++)
	{
		for(j=0;j<240;j++)
		{
			if(i>279)write_data(BLACK);
			else if(i>239)write_data(BLUE);
			else if(i>199)write_data(GREEN);
			else if(i>159)write_data(CYAN);
			else if(i>119)write_data(RED);
			else if(i>79)write_data(MAGENTA);
			else if(i>39)write_data(YELLOW);
			else write_data(WHITE);
		}
	}
}

// ************************************************* //
int main()
{
	PORTD = 0xF8;	    // pull-up on switches, all others off
	DDRD  = 0x0F;   	// VPP and led pins output

	DDRB = 0xFF;    	// low data port, all outputs
	DDRE = 0xFF;	    // high data port, all outputs
	
	PORTC = 0xFE;	    // all pins high
	DDRC  = 0xFE;	    // all outputs

	PORTB = 0x20;

	uint8 retry = 0;

	OSCCAL = 0x00;    //in order to operate some low speed card the initialization should run at lowest speed

	delay(0xffff);

	MMC_SD_Init();    //SPI initialize

	// Initialize the display
	initialization();

	write_command(0x0000);

	read_status(0);

	LCD_test();

	sd_restart: 	//label for "goto"

	while (MMC_SD_Reset())  // Initialize SD Card
	{
		retry++;
		if (retry > 254)
		{
			while (1)
			{
				goto sd_restart;				// try again
			}
		}
	}

	OSCCAL = 0xff;      //run at high speed for normal operation for SD Card

	delay(0xffff);      

	if (FAT_Init())   	//initialize file system - FAT16 and FAT32 are supported
	{
		while (1)
		{
			goto sd_restart;				// try again
		}
	}

	Search(PATH, &PictureInfo, &totalfiles);

	Picture_Slide_Show();

	while (1)
	{
	}
	return 0;

}
// ************************************************* //

